@@ -73,6 +73,11 @@ char *alloca ();
 # include <sys/stat.h>
 #endif
 
+#include <limits.h>
+#ifndef PATH_MAX
+# define PATH_MAX 1024
+#endif
+
 #if ! _LIBC && !HAVE_DECL_STPCPY && !defined stpcpy
 char *stpcpy ();
 #endif
@@ -349,18 +354,20 @@ process_entry (struct ftw_data *data, struct dir_data *dir, const char *name,
   struct STAT st;
   int result = 0;
   int flag = 0;
+  size_t new_buflen;
 
   if (name[0] == '.' && (name[1] == '\0'
 			 || (name[1] == '.' && name[2] == '\0')))
     /* Don't process the "." and ".." entries.  */
     return 0;
 
-  if (data->dirbufsize < data->ftw.base + namlen + 2)
+  new_buflen = data->ftw.base + namlen + 2;
+  if (data->dirbufsize < new_buflen)
     {
       /* Enlarge the buffer.  */
       char *newp;
 
-      data->dirbufsize *= 2;
+      data->dirbufsize = 2 * new_buflen;
       newp = (char *) realloc (data->dirbuf, data->dirbufsize);
       if (newp == NULL)
 	return -1;
@@ -406,11 +413,20 @@ process_entry (struct ftw_data *data, struct dir_data *dir, const char *name,
 		  /* Remember the object.  */
 		  && (result = add_object (data, &st)) == 0))
 	    {
-	      result = ftw_dir (data, &st);
+	      /* When processing a directory as part of a depth-first traversal,
+		 invoke the users callback function with type=FTW_DPRE
+		 just before processing any entry in that directory.
+		 And if the callback sets ftw.skip, then don't process
+		 any entries of the directory.  */
+	      if ((data->flags & FTW_DEPTH)
+		  && (result = (*data->func) (data->dirbuf, &st, FTW_DPRE,
+					      &data->ftw)) == 0
+		  && ! data->ftw.skip)
+		result = ftw_dir (data, &st);
 
 	      if (result == 0 && (data->flags & FTW_CHDIR))
 		{
-		  /* Change back to current directory.  */
+		  /* Change back to parent directory.  */
 		  int done = 0;
 		  if (dir->stream != NULL)
 		    if (__fchdir (dirfd (dir->stream)) == 0)
@@ -641,11 +657,7 @@ ftw_startup (const char *dir, int is_nftw, void *func, int descriptors,
   memset (data.dirstreams, '\0', data.maxdir * sizeof (struct dir_data *));
 
   dir_len = strlen (dir);
-#ifdef PATH_MAX
   data.dirbufsize = MAX (2 * dir_len, PATH_MAX);
-#else
-  data.dirbufsize = 2 * dir_len;
-#endif
   data.dirbuf = (char *) malloc (data.dirbufsize);
   if (data.dirbuf == NULL)
     return -1;
@@ -733,7 +745,15 @@ ftw_startup (const char *dir, int is_nftw, void *func, int descriptors,
 		result = add_object (&data, &st);
 
 	      if (result == 0)
-		result = ftw_dir (&data, &st);
+		{
+		  /* If we're doing a depth-first traversal, give the user
+		     a chance to prune the top-level directory.  */
+		  if ((flags & FTW_DEPTH)
+		      && (result = (*data.func) (data.dirbuf, &st, FTW_DPRE,
+						 &data.ftw)) == 0
+		      && ! data.ftw.skip)
+		    result = ftw_dir (&data, &st);
+		}
 	    }
 	  else
 	    {
