@@ -438,25 +438,6 @@ show_dev (const char *disk, const char *mount_point, const char *fstype,
   putchar ('\n');
 }
 
-/* Identify the directory, if any, that device
-   DISK is mounted on, and show its disk usage.  */
-
-static void
-show_disk (const char *disk)
-{
-  struct mount_entry *me;
-
-  for (me = mount_list; me; me = me->me_next)
-    if (STREQ (disk, me->me_devname))
-      {
-	show_dev (me->me_devname, me->me_mountdir, me->me_type,
-		  me->me_dummy, me->me_remote);
-	return;
-      }
-  /* No filesystem is mounted on DISK. */
-  show_dev (disk, (char *) NULL, (char *) NULL, 0, 0);
-}
-
 /* Return the root mountpoint of the filesystem on which FILE exists, in
    malloced storage.  FILE_STAT should be the result of stating FILE.  */
 static char *
@@ -522,9 +503,40 @@ done:
   return mp;
 }
 
+/* Identify the directory, if any, that device
+   DISK is mounted on, and show its disk usage.
+   STATP must be the result of `stat (DISK, STATP)'.  */
+
+static void
+show_disk (const char *disk, const struct stat *statp)
+{
+  struct mount_entry *me;
+  char *mount_point;
+
+  for (me = mount_list; me; me = me->me_next)
+    if (STREQ (disk, me->me_devname))
+      {
+	show_dev (me->me_devname, me->me_mountdir, me->me_type,
+		  me->me_dummy, me->me_remote);
+	return;
+      }
+
+  mount_point = find_mount_point (disk, statp);
+  if (!mount_point)
+    {
+      error (0, errno, "%s", quote (disk));
+      exit_status = EXIT_FAILURE;
+    }
+
+  /* No filesystem is mounted on DISK. */
+  show_dev (disk, mount_point, NULL, 0, 0);
+  if (mount_point)
+    free (mount_point);
+}
+
 /* Figure out which device file or directory POINT is mounted on
    and show its disk usage.
-   STATP is the results of `stat' on POINT.  */
+   STATP must be the result of `stat (POINT, STATP)'.  */
 static void
 show_point (const char *point, const struct stat *statp)
 {
@@ -656,7 +668,7 @@ static void
 show_entry (const char *path, const struct stat *statp)
 {
   if (S_ISBLK (statp->st_mode) || S_ISCHR (statp->st_mode))
-    show_disk (path);
+    show_disk (path, statp);
   else
     show_point (path, statp);
 }
